#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-

import os
import re
import json
from product_conf.devinfo import VOL_INFO
from product_conf import SUDO_CMD
from subprocess import Popen, PIPE
from pkgaddon.meimeikun_config import FISCAL_YEAR_FOLDER_RE
from fal_info import (FAL_SERVICECTRL_PATH, ReqTypes)

LOG_SHARE = "/mnt/sataraid1/share/LAN DISK Log/Meimeikun"
KEEP_SHARE = "/mnt/sataraid1/share/meimei_ro$"

MEIMEIKUN_CONFIG_COMMAND = "/usr/local/bin/meimeikun/meimeikun_config"
OPERATION_LOAD = "load"
MEIMEIKUN_HANDLER_COMMAND = "/mnt/data/libexec/meimeikun/inotify_handler"
MEIMEIKUN_INOTIFYD_CMD = "/etc/init.d/meimeikun-inotifyd"
OPTION_COMPARE = "--compare"
SHARE_ROOT = os.path.join(VOL_INFO["sataraid1"]["mnt_point"],
                          VOL_INFO["sataraid1"]["share_root"])


def _encode_str(text):
    if isinstance(text, unicode):
        return text.encode('utf-8')
    else:
        return text


def _load_meimekun_config(sudo=True):
    cmd = [MEIMEIKUN_CONFIG_COMMAND, OPERATION_LOAD]
    conifg = "{}"
    try:
        conifg = __execute_cmd(cmd, sudo)
    except Exception as err:
        pass
    return json.loads(conifg)


def _copy_to_protected_share(src_dir, sudo=True):
    cmd = [MEIMEIKUN_HANDLER_COMMAND, src_dir, OPTION_COMPARE]
    try:
        __execute_cmd(cmd, sudo)
    except Exception as err:
        pass
    return


def __execute_cmd(cmd, sudo=False):
    if sudo:
        cmd = [SUDO_CMD] + cmd

    proc = Popen(cmd, stdout=PIPE, stderr=PIPE)
    stdout, stderr = proc.communicate()

    if proc.returncode != 0:
        raise Exception(stderr)

    return stdout


def _chmod_dirs(dir_path, mode):
    for _dir in find_all_dirs(dir_path):
        os.chmod(_dir, mode)


def find_all_dirs(dir_path):
    for curDir, dirs, files in os.walk(dir_path):
        yield curDir


def _chmod_fiscal_year_dirs(share_name, mode):
    path = os.path.join(SHARE_ROOT, share_name)
    for sub_dir in list_dirs(path):
        if re.match(FISCAL_YEAR_FOLDER_RE, sub_dir):
            os.chmod(os.path.join(path, sub_dir), 0o777)


def list_dirs(dir_path):
    for _dir in os.listdir(dir_path):
        if os.path.isfile(os.path.join(dir_path, _dir)):
            continue
        yield _dir


def restart_inotifyd():
    __execute_cmd([MEIMEIKUN_INOTIFYD_CMD, "stop"], sudo=True)
    __execute_cmd([MEIMEIKUN_INOTIFYD_CMD, "start"], sudo=True)


def __enable_keep_share():
    """Enable keep share."""

    from pal_info import (PAL_SAMBA_PATH, )
    MEIMEIKUN_CONFIG_DIR = "/mnt/data/conf/meimeikun"
    RO_SUFFIX = "ro"
    RW_SUFFIX = "rw"

    if os.path.isdir(MEIMEIKUN_CONFIG_DIR) is False:
        raise Exception

    for suffix in [RO_SUFFIX, RW_SUFFIX]:
        part_file = "meimei_ro$" + os.extsep + suffix
        part_path = os.path.join(MEIMEIKUN_CONFIG_DIR, part_file)
        samba_part_path = os.path.join("/mnt/data/conf/samba/sataraid1",
                                       part_file)
        if os.path.islink(samba_part_path):
            os.unlink(samba_part_path)

        os.symlink(part_path, samba_part_path)

    __send_request(PAL_SAMBA_PATH, ReqTypes.UPDATE, """<samba/>""")


def enable_keep_share():
    """Enable keep share."""

    SERVICECTRL_SHARE_REQ_XML = """<servicectrl>
        <categorylist>
            <category><name>share</name></category>
        </categorylist>
    </servicectrl>
    """

    __send_request(FAL_SERVICECTRL_PATH, ReqTypes.SUSPEND,
                   SERVICECTRL_SHARE_REQ_XML)

    __enable_keep_share()

    __send_request(FAL_SERVICECTRL_PATH, ReqTypes.RESUME,
                   SERVICECTRL_SHARE_REQ_XML)


def __send_request(target, type, body=""):
    """Send request entry to "target"."""

    from ionas import request
    entry = request.Entry(type, body)
    return request.Request(target).request(entry).get_entry()

###############################################################################
# main routine
###############################################################################


try:
    share_list = []
    restart_inotifyd()

    # Enable samba access.
    enable_keep_share()

    # Set the permissions on the log share to 777.
    if os.path.exists(LOG_SHARE):
        _chmod_dirs(LOG_SHARE, 0o777)

    # Set the permissions on the keep share to 777.
    if os.path.exists(KEEP_SHARE):
        _chmod_dirs(KEEP_SHARE, 0o777)

    config = _load_meimekun_config()
    timestamp_share = config.get("timestamp_share", None)
    prekeep_share = config.get("prekeep_share", None)

    if timestamp_share:
        timestamp_share = _encode_str(timestamp_share)
        share_list.append(timestamp_share)
    if prekeep_share:
        prekeep_share = _encode_str(prekeep_share)
        share_list.append(prekeep_share)

    # Set the permissions on the fiscal year folder to 777.
    for share in share_list:
        _chmod_fiscal_year_dirs(share, 0o777)

    if prekeep_share is None:
        exit(0)

    path = os.path.join(SHARE_ROOT, prekeep_share)
    for _dir in list_dirs(path):
        _copy_to_protected_share(os.path.join(path, _dir))

    exit(0)
except Exception as err:
    exit(1)
