#!/usr/bin/env python
# -*- coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import absolute_import
from __future__ import division

import sys

from ionas.request import Request, Entry, EntryList, XmlConfNode
from product_conf import SUDO_CMD

__all__ = ['buzzer', 'OK', 'ERROR']


OK = 'ok'
ERROR = 'error'


def send_entry(falcmd, entry, raise_exception=False):
    try:
        entrylist = EntryList()
        entrylist.append_entry(entry)
        req = Request(falcmd, filter_cmd=SUDO_CMD)
        res = req.request(entrylist)
        return res.get_entrylist().get_entry()
    except Exception as e:
        if raise_exception:
            raise
        return None


def buzzer(t, raise_exception=False):
    """
    buzzer(OK)
    buzzer(ERROR)
    """
    try:
        from fal_info import FAL_BUZZER_PATH
        from fal_info import Results
        entry = Entry('play', '<buzzer><melody type="{t}" /></buzzer>'.format(
            t=t,
        ))
        entry = send_entry(
            FAL_BUZZER_PATH,
            entry,
            raise_exception=raise_exception,
        )
        if entry.get_result() not in (Results.OK,):
            raise
    except Exception as e:
        if raise_exception:
            raise


if __name__ == '__main__':
    # NOTE: python /path/to/this/file ok
    buzzer(sys.argv[1])
