# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django import forms
from project.view import FuncView, MenuView
from root.modules.form import BaseForm
from pkgaddon.meimeikun_config import KEEP_SHARE_NAME

import os
import datetime
from ionas.request import Entry
from . import fields
from .. import _utils
from fal_info import (FAL_SHARES_PATH,)
from django.core.exceptions import ValidationError


def _get_exist_shares():
    shares = []
    try:
        entry = Entry('load', '<shares />')
        entry = _utils.send_entry(FAL_SHARES_PATH, entry)

        for share_node in entry.get().get_node_list('sharelist/share'):
            if share_node.get_attr_value('undeletable', False):
                continue
            share_name = share_node.get_node('name').get_text()
            shares.append(share_name)
    except Exception as err:
        pass
    return shares


class MeimeikunSettingForm(BaseForm):
    def __init__(self, **kwargs):
        super(MeimeikunSettingForm, self).__init__(**kwargs)

    def clean(self):
        cleaned_data = self.cleaned_data
        closing_month = cleaned_data.get('closing_month')
        closing_day = cleaned_data.get('closing_day')

        if not self.validate_date(int(closing_month), int(closing_day)):
            self._errors['closing_day'] = self.error_class(["存在しない日付が指定されています。"])

        return cleaned_data

    def validate_date(self, month, day):
        try:
            year = datetime.datetime.today().year
            datetime.datetime(year, month, day)
        except Exception as err:
            return False

        if month == 2 and day == 29:
            return False

        return True

    fiscal_year = fields.MeimeikunFiscalYearField()
    closing_month = fields.MeimeikunClosingMonthField()
    closing_day = fields.MeimeikunClosingDayField()


class MeimeikunSettingView(FuncView):
    views = ['apply']

    def _get_current_config(self):
        config = _utils.load_config()
        current_config = {}
        current_config["fiscal_year"] = config["fiscal_year"]
        current_config["closing_month"] = config["closing_date"]["month"]
        current_config["closing_day"] = config["closing_date"]["day"]
        current_config["timestamp_share"] = config["timestamp_share"]
        current_config["prekeep_share"] = config["prekeep_share"]
        current_config["keep_share"] = KEEP_SHARE_NAME

        return current_config

    def view(self, request):
        config = self._get_current_config()
        init_form = MeimeikunSettingForm(initial=config)
        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'init_form': init_form,
            'config': config,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_apply(self, request):
        init_form = MeimeikunSettingForm(data=request.POST)
        form = init_form
        with self.processing(constname='ERR_MEIMEIKUN_STATE_CHANGE') as proc:
            proc.break_if_state_change()
            if form.is_valid():
                cleaned_data = form.cleaned_data
                config_dict = {
                              "fiscal_year": cleaned_data["fiscal_year"],
                              "closing_date": {"day": int(cleaned_data["closing_day"]),
                                               "month": int(cleaned_data["closing_month"])}
                }
                succeeded, message = _utils.update_config(config_dict)
                proc.result(succeeded)
                self.set_message(message)
        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'init_form': form,
            'config': self._get_current_config(),
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def make_config_dict(self, cleaned_data):
        config_dict = {
            "fiscal_year": cleaned_data["fiscal_year"],
            "closing_date": {
                "month": int(cleaned_data["closing_month"]),
                "day": int(cleaned_data["closing_day"])
            },
            "timestamp_share": cleaned_data["timestamp_share"],
            "prekeep_share": cleaned_data["prekeep_share"],
        }
        return config_dict
