# -*- coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import absolute_import
from __future__ import division

import re
import datetime

from django.utils.translation import ugettext as _
from django import forms
from django.core.exceptions import ValidationError

from project.forms.fields.extras.enable import EnableField
from project.forms.fields.select import SelectField

from .disk import Disk


class MeimeikunYearField(forms.IntegerField):

    def __init__(self, *args, **kwargs):
        kwargs['label'] = kwargs.get('label', '事業年度')
        kwargs['initial'] = kwargs.get('initial', datetime.datetime.now().year)
        kwargs['min_value'] = kwargs.get('min_value', 2000)
        kwargs['max_value'] = kwargs.get('max_value', 2099)
        super(MeimeikunYearField, self).__init__(*args, **kwargs)


class MeimeikunVolumeLabelField(forms.CharField):

    def __init__(self, *args, **kwargs):
        kwargs['label'] = kwargs.get('label', 'ボリュームラベル')
        super(MeimeikunVolumeLabelField, self).__init__(*args, **kwargs)

    def validate(self, value):
        super(MeimeikunVolumeLabelField, self).validate(value)
        if not Disk.is_valid_volumelabel(value):
            raise ValidationError(_(u"Enter a valid value."))
        # if len(value.encode('UTF-16')) > 32:
        #     raise ValidationError('半角32文字または全角10文字以内で入力してください。')


class MeimeikunArchiveScheduleEnabledField(EnableField):

    def __init__(
        self,
        label=u"スケジュール書き込み",
        *args, **kwargs):
        super(MeimeikunArchiveScheduleEnabledField, self).__init__(
            label=label,
            *args, **kwargs
        )


class MeimeikunArchiveScheduleDayOfMonthField(SelectField):

    def __init__(self,
                 choices=[(h, h) for h in range(1, 32)],
                 label=u"日",
                 *args, **kwargs):

        super(MeimeikunArchiveScheduleDayOfMonthField, self).__init__(
            choices=choices,
            label=label,
            *args, **kwargs
        )

    def clean(self, value):
        value = super(MeimeikunArchiveScheduleDayOfMonthField, self).clean(value)
        return int(value)


class MeimeikunArchiveScheduleTimeOfDayField(forms.TimeField):

    def __init__(self,
                 label=u"時刻",
                 input_formats=['%H:%M'],
                 initial=datetime.time(23, 55),
                 *args, **kwargs):
        super(MeimeikunArchiveScheduleTimeOfDayField, self).__init__(
            label=label,
            input_formats=input_formats,
            initial=initial,
            *args, **kwargs
        )
