# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

from django.utils.translation import ugettext as _
from django import forms
from datetime import datetime
from root.modules.form.fields.select import SelectField
from project.forms.fields.extras.share import ShareNameField
from project.forms.fields.extras.enable import EnableField

from fal_validinfo import ShareName


class MeimeikunFiscalYearField(forms.IntegerField):

    def __init__(self, *args, **kwargs):
        kwargs['label'] = kwargs.get('label', '事業年度')
        kwargs['initial'] = kwargs.get('initial', datetime.now().year)
        kwargs['min_value'] = kwargs.get('min_value', 2000)
        kwargs['max_value'] = kwargs.get('max_value', 2099)
        kwargs['required'] = kwargs.get('required', True)
        super(MeimeikunFiscalYearField, self).__init__(*args, **kwargs)


class MeimeikunClosingMonthField(SelectField):

    def __init__(self, *args, **kwargs):
        kwargs['label'] = kwargs.get('label', _(u'month'))
        kwargs['initial'] = kwargs.get('initial', 12)
        kwargs['required'] = kwargs.get('required', True)
        kwargs['choices'] = kwargs.get('choices', ((h, h) for h in range(1, 13)))
        super(MeimeikunClosingMonthField, self).__init__(*args, **kwargs)


class MeimeikunClosingDayField(SelectField):

    def __init__(self, *args, **kwargs):
        kwargs['label'] = kwargs.get('label', _(u'day'))
        kwargs['initial'] = kwargs.get('initial', 31)
        kwargs['required'] = kwargs.get('required', True)
        kwargs['choices'] = kwargs.get('choices', ((h, h) for h in range(1, 32)))
        super(MeimeikunClosingDayField, self).__init__(*args, **kwargs)


class MeimeikunShareNameSetField(forms.CharField):

    def __init__(self, *args, **kwargs):
        kwargs['label'] = kwargs.get('label', '')
        kwargs['max_length'] = kwargs.get('max_length', ShareName.MAX_LENGTH)
        super(MeimeikunShareNameSetField, self).__init__(*args, **kwargs)


class MeimeikunArchiveScheduleEnabledField(EnableField):

    def __init__(self, *args, **kwargs):
        kwargs['label'] = kwargs.get('label', u'スケジュール書き込み')
        kwargs['initial'] = kwargs.get('initial', True)
        super(MeimeikunArchiveScheduleEnabledField, self).__init__(*args, **kwargs)


class MeimeikunArchiveScheduleDayOfMonthField(SelectField):

    def __init__(self, *args, **kwargs):
        kwargs['label'] = kwargs.get('label', _(u'day'))
        kwargs['initial'] = kwargs.get('initial', 31)
        kwargs['required'] = kwargs.get('required', True)
        # kwargs['choices'] = kwargs.get('choices', [(d, '{0} 日'.format(d)) for d in range(1, 31)])
        kwargs['choices'] = kwargs.get('choices', ((h, h) for h in range(1, 32)))
        super(MeimeikunArchiveScheduleDayOfMonthField, self).__init__(*args, **kwargs)


class MeimeikunArchiveScheduleTimeOfDayField(forms.TimeField):

    def __init__(self, *args, **kwargs):
        kwargs['label'] = kwargs.get('label', 'スケジュールの時刻')
        # kkwargs['initial'] = kwargs.get('initial', datetime.time(23, 55))  # FIXME
        kwargs['required'] = kwargs.get('required', True)
        kwargs['input_formats'] = kwargs.get('input_formats', ['%H:%M'])
        super(MeimeikunArchiveScheduleTimeOfDayField, self).__init__(*args, **kwargs)


class MeimeikunVolumeLabelField(forms.CharField):

    def __init__(self, *args, **kwargs):
        kwargs['label'] = kwargs.get('label', 'ボリュームラベル')
        super(MeimeikunVolumeLabelField, self).__init__(*args, **kwargs)
